#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "psg.h"

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

int parse_options(int, char **, PSG_Settings *);

int main(int argc, char *argv[])
{
  raw_buffer *raw;
  YM_Header *head;
  PSG_Settings set;
  int arg, error=0;

  arg = parse_options(argc, argv, &set);
  open_audio(&set);

  for( ; arg < argc ; arg++) 
    {
      if(set.verbose)
	fprintf(stderr, "Playing %s\n", argv[arg]);

      raw = load_file(argv[arg]);
      if(!raw)
	{
	  fprintf(stderr, "Unable to load file: %s\n", argv[arg]);
	  error++;
	  continue;
	}
      
      head = psg_build_header(raw->buffer, raw->size, &set);
      if(!head)
	{
	  fprintf(stderr, "Unable to get proper header\n");
	  error++;
	  continue;
	}
      
      do
	{
	  psg_play(head, &set);
	}
      while(set.repeat);

      free(raw->buffer);
    }  

  return 0;
}

int *get_left_channels(char *data)
{
  char *tmp;
  int i, *left;
  
  tmp = strdup(data);
  
  left = (int *) malloc(sizeof(int) * strlen(tmp));

  for(i = 0; i < strlen(tmp); i++) {
    if(tmp[i] == ',') break;
    if((tmp[i] >= '0') && (tmp[i] <= '9')) {
      left[0]++;
      left[left[0]] = (int) (tmp[i] - '0');
    }
  }
  free(tmp);
  return left;
}

int *get_right_channels(char *data)
{
  char *tmp;
  int i, *right;
  
  tmp = strdup(data);
 
  right = (int *) malloc(sizeof(int) * strlen(tmp));
 
  for(i = 0; i < strlen(tmp); i++) {
    if(tmp[i] == ',') break;
  }
  for(; i < strlen(tmp); i++) {
    if((tmp[i] >= '0') && (tmp[i] <= '9')) {
      right[0]++;
      right[right[0]] = (int) (tmp[i] - '0');
    }
  }
  free(tmp);
  return right;
}

int parse_options(int argc, char *argv[], PSG_Settings *set)
{
  int arg;

#ifdef HAVE_GETOPT_LONG
  struct option long_options[] = { 
    { "echo", no_argument, NULL, 'e' },
    { "verbose", no_argument, NULL, 'v' },
    { "quiet", no_argument, NULL, 'q' },
    { "stereo", no_argument, NULL, 's' },
    { "channels", required_argument, NULL, 'c' },
    { "surround", no_argument, NULL, 'u' },
    { "help", no_argument, NULL, 'h' },
    { "output", required_argument, NULL, 'o' },
    { "repeat", no_argument, NULL, 'r' },
    { "version", no_argument, NULL, 'V' },
    { NULL, 0, NULL, 0 } };
#endif

  char *short_options = "o:c:evqsruhV";

  /* default settings */
  set->repeat = set->echo = set->verbose = set->quiet = set->stereo = 
    set->surround = set->channels = 0;
  set->output = NULL;

  while((arg=getopt_long(argc, argv, short_options, long_options, NULL)) != EOF) {
    switch(arg) {
    case 'e':
      set->echo = 1;
      break;
    case 'r':
      set->repeat = 1;
      break;
    case 'v':
      set->verbose = 1;
      break;
    case 's':
      set->stereo = 1;
      break;
    case 'c':
      set->channels = 1;
      set->chleft = get_left_channels(optarg);
      set->chright = get_right_channels(optarg);
      break;
    case 'u':
      set->surround = 1;
      break;
    case 'q':
      set->quiet = 1;
      break;
    case 'o':
      set->output = optarg;
      break;
    case 'h':
      fprintf(stderr,"Usage: %s [OPTION(S)] <filename>\n"
	      "\n"
	      "valid options are:\n"
	      "-e, --echo            use echo effects on the sound\n"
	      "-q, --quiet           don't print anything at all (inactive)\n"
	      "-v, --verbose         print informational messages\n"
	      "-s, --stereo          use stereo output (L = A+B, R = B+C)\n"
	      "-c, --channels        choose stereo setup (only with -s)\n"
	      "-u, --surround        use dolby surround (inactive)\n"
	      "-h, --help            print this text and exit\n"
	      "-o, --output FILE     writes output to FILE\n"
	      "-r, --repeat          repeat first song forever\n"
	      "-V, --version         print version and exit\n"
	      "\n"
              ,argv[0]);
      exit(1);
      break;
    case 'V':
      fprintf(stderr,"PSG-Play version 0.7.0\n");
      exit(1);
      break;
    default:
    }
  }

  if(!set->channels && set->stereo) {
    set->chleft = (int *) malloc(sizeof(int) * 4);
    set->chright = (int *) malloc(sizeof(int) * 4);
    set->chleft[0] = 2;
    set->chleft[1] = 1;
    set->chleft[2] = 2;
    
    set->chright[0] = 2;
    set->chright[1] = 2;
    set->chright[2] = 3;
  }
  if(!set->stereo) {
    set->chleft = (int *) malloc(sizeof(int) * 4);
    set->chright = (int *) malloc(sizeof(int) * 4);
    set->chleft[0] = 3;
    set->chleft[1] = 1;
    set->chleft[2] = 2;
    set->chleft[3] = 3;
    
    set->chright[0] = 3;
    set->chright[1] = 1;
    set->chright[2] = 2;
    set->chright[3] = 3;
  }

  return optind;
}
